<?php

namespace Axima\Pays\Controller\Adminhtml\Settings;

use Axima\Pays\Block\Adminhtml\AximaPaysBlock;
use Axima\Pays\Utils\Database;
use Axima\Pays\Utils\RedirectException;
use DateTime;
use Exception;
use Magento\Backend\App\Action;
use Magento\Cms\Api\PageRepositoryInterface;
use Magento\Framework\Api\SearchCriteria;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Phrase;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Framework\Url;

/**
 * @author Tomáš Blatný
 */
class Index extends Action
{

	const DOMAIN = 'axima_pays';
	const TABLE_PAYMENTS = 'axima_pays_payments';
	const CONFIG_PATH = __DIR__ . '/../../../../../../../var/config_axima_pays.json';

	const STATUS_INITIATED = 0;
	const STATUS_PAID = 1;
	const STATUS_ERROR = 2;

	/** @var Database */
	private $database;

	/** @var TimezoneInterface */
	private $timezone;

	public function execute()
	{
		$this->database = new Database($this->_objectManager->get('\Magento\Framework\App\ResourceConnection'));
		$this->timezone = $this->_objectManager->get('\Magento\Framework\Stdlib\DateTime\TimezoneInterface');

		try {
			$this->resolveRequest();
		} catch (RedirectException $e) {
			/** @var Redirect $redirect */
			$redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
			$redirect->setUrl($e->getUrl());
			return $redirect;
		}

		$this->_view->loadLayout();
		$this->_view->renderLayout();
		return NULL;
	}


	public function redirect($action = NULL, $other = NULL)
	{
		throw new RedirectException($this->link($action, $other));
	}


	public function link($action = NULL, $other = NULL)
	{
		$url = $this->getUrl('Pays/settings');
		if ($action === NULL && $other === NULL) {
			return $url;
		}
		return $url . '?' . http_build_query(['action' => $action]) . $other;
	}


	public function resolveRequest()
	{
		$action = isset($_GET['action']) ? $_GET['action'] : 'default';
		if (!in_array($action, array('default', 'logs', 'settings'), TRUE)) {
			$this->redirect('default');
		}
		$method = 'render' . ucfirst($action);
		$data = array();
		if (method_exists($this, $method)) {
			$data = (array) $this->$method();
		}
		$that = $this;

		$confirmationUrl = $this->_objectManager->get('Magento\Store\Model\StoreManagerInterface')->getStore()->getBaseUrl();
		if (strpos($confirmationUrl, '?') !== FALSE) {
			$confirmationUrl .= '&paysConfirm=1';
		} else {
			$confirmationUrl .= '?paysConfirm=1';
		}
		AximaPaysBlock::setTemplateData(array(
			'page' => $action,
			'date' => function ($date, $format = 'j.n.Y G:i:s') use ($that) {
				try {
					$dateTimeZone = new \DateTimeZone($that->timezone->getDefaultTimezone());
					$date = new DateTime($date, $dateTimeZone);
					return $date->format($format);
				} catch (Exception $e) {
					return '-';
				}
			},
			'domain' => self::DOMAIN,
			'confirmationUrl' => $confirmationUrl,
			'link' => function ($action, $other = NULL) use ($that) {
				return $that->link($action, $other);
			},
		) + $data);
	}


	private function renderDefault()
	{
		$lastWeek = new DateTime('-1 week');
		$lastWeek = $lastWeek->format('Y-m-d G:i:s');
		return array(
			'totalInitiated' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` IN (%d, %d)', array(self::STATUS_INITIATED, self::STATUS_PAID)),
			'totalPaid' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` = %d', array(self::STATUS_PAID)),
			'lastWeekInitiated' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` IN (%d, %d) AND `date_initiated` > %s', array(self::STATUS_INITIATED, self::STATUS_PAID, $lastWeek)),
			'lastWeekPaid' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` = %d AND `date_initiated` > %s', array(self::STATUS_PAID, $lastWeek)),
			'totalErrors' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` = %d', array(self::STATUS_ERROR)),
			'lastWeekErrors' => $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS, 'WHERE `status` = %d AND `date_initiated` > %s', array(self::STATUS_ERROR, $lastWeek)),
		);
	}


	private function renderLogs()
	{
		$page = (int) $this->getQuery('list', 1);
		$count = (int) $this->database->selectOne('COUNT(*)', self::TABLE_PAYMENTS);
		$perPage = 30;
		$pageCount = (int) ceil($count / $perPage);
		if ($page < 1) {
			$page = 1;
		}
		if ($page > $pageCount) {
			$page = $pageCount;
		}
		$pages = array(1, $pageCount, $page, $page - 1, $page - 2, $page + 1, $page + 2);
		if ($pageCount > 5) {
			for ($i = 1; $i <= $pageCount; $i += ($pageCount / 5)) {
				$pages[] = round($i);
			}
		}
		$pages = array_unique(array_filter($pages, function ($item) use ($pageCount) {
			return ($item >= 1) && ($item <= $pageCount);
		}));
		sort($pages);
		$results = array();
		if ($count) {
			$results = $this->database->select(self::TABLE_PAYMENTS, 'ORDER BY `date_initiated` DESC LIMIT ' . $perPage . ' OFFSET ' . ($page - 1) * $perPage);
		}

		return array(
			'payments' => $results,
			'pages' => $pages,
			'list' => $page,
			'maxPage' => $pageCount,
		);
	}


	private function renderSettings()
	{
		if (isset($_POST['_submit'])) {
			$password = trim($this->getPost('hash-password'));
			$update = array(
				'merchantId' => trim($this->getPost('merchant-id')),
				'shopId' => trim($this->getPost('shop-id')),
				'success-url' => $this->getPageUrl(trim($this->getPost('success-url'))),
				'error-url' => $this->getPageUrl(trim($this->getPost('error-url'))),
			);
			if ($password) {
				$update['hashPassword'] = $password;
			}
			self::updateOptions($update);
		}

		$pages = array();

		$pages[''] = ' - Choose - ';

		/** @var PageRepositoryInterface $pageRepository */
		$pageRepository = $this->_objectManager->get('\Magento\Cms\Api\PageRepositoryInterface');
		/** @var Url $urlInterface */
		$urlInterface = $this->_objectManager->get('\Magento\Framework\Url');
		$items = $pageRepository->getList(new SearchCriteria);

		foreach ($items->getItems() as $item) {
			if ($item['active']) {
				$pages[$this->getPageUrl($urlInterface->getUrl($item['identifier']))] = $item['title'];
			}
		}

		return array(
			'settings' => self::getOptions(),
			'pages' => $pages,
		);
	}


	private function getPageUrl($pageUrl)
	{
		if (!$pageUrl) {
			return $pageUrl;
		}
		if (($pos = strpos($pageUrl, '?SID=')) !== FALSE) {
			return substr($pageUrl, 0, $pos);
		}
		return $pageUrl;
	}


	// -----------------------------



	private static function updateOptions(array $options)
	{
		$current = self::getOptions();
		foreach ($options as $key => $option) {
			$current[$key] = $option;
		}
		if (!@file_put_contents(self::CONFIG_PATH, json_encode($current))) {
			throw new FileSystemException(new Phrase('Cannot write to config file, please check if Magento has write access to var directory.'));
		}
	}


	/**
	 * @return array
	 */
	private static function getOptions()
	{
		$config = @file_get_contents(self::CONFIG_PATH);
		if (!$config) {
			$config = array();
		} else {
			$config = json_decode($config, JSON_OBJECT_AS_ARRAY);
		}
		return $config;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private static function getOption($name, $default = NULL)
	{
		$options = self::getOptions();
		return isset($options[$name]) ? $options[$name] : $default;
	}

	private static function getHashPassword()
	{
		return self::getOption('hashPassword', '');
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private function getPost($name, $default = NULL)
	{
		return isset($_POST[$name]) && $_POST[$name] ? $_POST[$name] : $default;
	}


	private function getQuery($name, $default = NULL)
	{
		return isset($_GET[$name]) && $_GET[$name] ? $_GET[$name] : $default;
	}

}
